@@ -495,13 +495,14 @@ def save_context_core(
         )
 
         logger.important(
-            f"finished Run('{run.uid[:8]}') after {formatted_run_time} at {format_field_value(run.finished_at)}"
+            f"finished Run('{run.uid}') after {formatted_run_time} at {format_field_value(run.finished_at)}"
         )
     if ln_setup.settings.instance.is_on_hub:
         instance_slug = ln_setup.settings.instance.slug
         if save_source_code_and_report:
+            ui_url = ln_setup.settings.instance.ui_url
             logger.important(
-                f"go to: https://lamin.ai/{instance_slug}/transform/{transform.uid}"
+                f"go to: {ui_url}/{instance_slug}/transform/{transform.uid}"
             )
         if finished_at and not from_cli and save_source_code_and_report:
             thing = "notebook" if (is_ipynb or is_r_notebook) else "script"
