@@ -23,7 +23,8 @@ from ..errors import (
     TrackNotCalled,
     UpdateContext,
 )
-from ..models import Run, Transform, format_field_value
+from ..models import Run, SQLRecord, Transform, format_field_value
+from ..models._feature_manager import infer_feature_type_convert_json
 from ..models._is_versioned import bump_version as bump_version_function
 from ..models._is_versioned import (
     increment_base62,
@@ -235,6 +236,22 @@ class LogStreamTracker:
         self.original_excepthook(exc_type, exc_value, exc_traceback)
 
 
+def serialize_params_to_json(params: dict) -> dict:
+    serialized_params = {}
+    for key, value in params.items():
+        if isinstance(value, SQLRecord):
+            value = f"{value.__class__.__get_name_with_module__()}[{value.uid}]"
+        else:
+            dtype, _, _ = infer_feature_type_convert_json(key, value)
+            if (dtype == "?" or dtype.startswith("cat")) and dtype != "cat ? str":
+                logger.warning(
+                    f"skipping param {key} because dtype not JSON serializable"
+                )
+                continue
+        serialized_params[key] = value
+    return serialized_params
+
+
 class Context:
     """Run context.
 
@@ -325,6 +342,7 @@ class Context:
         project: str | Project | None = None,
         space: str | Space | None = None,
         branch: str | Branch | None = None,
+        features: dict | None = None,
         params: dict | None = None,
         new_run: bool | None = None,
         path: str | None = None,
@@ -343,7 +361,8 @@ class Context:
                 Default: the `"all"` space. Note that bionty entities ignore this setting and always get written to the `"all"` space.
                 If you want to manually move entities to a different space, set the `.space` field (:doc:`docs:access`).
             branch: A branch (or its `name` or `uid`) on which to store records.
-            params: A dictionary of parameters to track for the run.
+            features: A dictionary of features & values to track for the run.
+            params: A dictionary of params & values to track for the run.
             new_run: If `False`, loads the latest run of transform
                 (default notebook), if `True`, creates new run (default non-notebook).
             path: Filepath of notebook or script. Only needed if it can't be
@@ -465,10 +484,14 @@ class Context:
                 transform_exists = Transform.filter(id=transform.id).first()
             if transform_exists is None:
                 transform.save()
-                self._logging_message_track += f"created Transform('{transform.uid}')"
+                self._logging_message_track += (
+                    f"created Transform('{transform.uid}', key='{transform.key}')"
+                )
                 transform_exists = transform
             else:
-                self._logging_message_track += f"loaded Transform('{transform.uid}')"
+                self._logging_message_track += (
+                    f"loaded Transform('{transform.uid}', key='{transform.key}')"
+                )
             self._transform = transform_exists
 
         if new_run is None:  # for notebooks, default to loading latest runs
@@ -493,25 +516,26 @@ class Context:
             if run is not None:  # loaded latest run
                 run.started_at = datetime.now(timezone.utc)  # update run time
                 run._status_code = -2  # re-started
-                self._logging_message_track += f", re-started Run('{run.uid[:8]}...') at {format_field_value(run.started_at)}"
+                self._logging_message_track += f", re-started Run('{run.uid}') at {format_field_value(run.started_at)}"
 
         if run is None:  # create new run
-            run = Run(  # type: ignore
-                transform=self._transform,
-                params=params,
-            )
+            run = Run(transform=self._transform)
             run.started_at = datetime.now(timezone.utc)
             run._status_code = -1  # started
-            self._logging_message_track += f", started new Run('{run.uid[:8]}...') at {format_field_value(run.started_at)}"
+            self._logging_message_track += f", started new Run('{run.uid}') at {format_field_value(run.started_at)}"
         # can only determine at ln.finish() if run was consecutive in
         # interactive session, otherwise, is consecutive
         run.is_consecutive = True if is_run_from_ipython else None
-        # need to save in all cases
-        run.save()
         if params is not None:
-            run.features.add_values(params)
+            run.params = serialize_params_to_json(params)
             self._logging_message_track += "\n→ params: " + ", ".join(
-                f"{key}={value}" for key, value in params.items()
+                f"{key}={value}" for key, value in run.params.items()
+            )
+        run.save()  # need to save now
+        if features is not None:
+            run.features.add_values(features)
+            self._logging_message_track += "\n→ features: " + ", ".join(
+                f"{key}={value}" for key, value in features.items()
             )
         self._run = run
         track_python_environment(run)
@@ -835,7 +859,9 @@ class Context:
                 reference_type=transform_ref_type,
                 type=transform_type,
             ).save()
-            self._logging_message_track += f"created Transform('{transform.uid}')"
+            self._logging_message_track += (
+                f"created Transform('{transform.uid}', key='{transform.key}')"
+            )
         else:
             uid = transform.uid
             # transform was already saved via `finish()`
@@ -874,9 +900,7 @@ class Context:
                     if transform_hash != transform.hash:
                         bump_revision = True
                     else:
-                        self._logging_message_track += (
-                            f"loaded Transform('{transform.uid}')"
-                        )
+                        self._logging_message_track += f"loaded Transform('{transform.uid}', key='{transform.key}')"
                 if bump_revision:
                     change_type = (
                         "re-running notebook with already-saved source code"
@@ -890,7 +914,9 @@ class Context:
                         f'✗ {change_type}, please update the `uid` argument in `track()` to "{uid[:-4]}{increment_base62(uid[-4:])}"'
                     )
             else:
-                self._logging_message_track += f"loaded Transform('{transform.uid}')"
+                self._logging_message_track += (
+                    f"loaded Transform('{transform.uid}', key='{transform.key}')"
+                )
         self._transform = transform
 
     def _finish(self, ignore_non_consecutive: None | bool = None) -> None:
