@@ -289,13 +289,7 @@ class Curator:
 
         artifact_info = ""
         if self._artifact is not None:
-            artifact_uid = getattr(self._artifact, "uid", str(self._artifact))
-            short_uid = (
-                str(artifact_uid)[:8] + "..."
-                if len(str(artifact_uid)) > 8
-                else str(artifact_uid)
-            )
-            artifact_info = f", artifact: {colors.italic(short_uid)}"
+            artifact_info = f", artifact: {colors.italic(self._artifact.uid)}"
 
         return (
             f"{cls_name}{artifact_info}(Schema: {schema_str}{extra_info}{status_str})"
@@ -337,7 +331,7 @@ class SlotsCurator(Curator):
     def validate(self) -> None:
         """{}"""  # noqa: D415
         for slot, curator in self._slots.items():
-            logger.info(f"validating slot {slot} ...")
+            logger.debug(f"validating slot {slot} ...")
             curator.validate()
         # set _is_validated to True as no slot raised an error
         self._is_validated = True
@@ -403,6 +397,16 @@ class SlotsCurator(Curator):
         )
 
 
+def convert_dict_to_dataframe_for_validation(d: dict, schema: Schema) -> pd.DataFrame:
+    """Convert a dictionary to a DataFrame for validation against a schema."""
+    df = pd.DataFrame([d])
+    for feature in schema.members:
+        if feature.dtype.startswith("cat"):
+            if feature.name in df.columns:
+                df[feature.name] = pd.Categorical(df[feature.name])
+    return df
+
+
 # This is also currently used as DictCurator by flattening dictionaries into wide DataFrames.
 # Such an approach was never intended and there is room for a DictCurator in the future.
 # For more context, read https://laminlabs.slack.com/archives/C07DB677JF6/p1753994077716099 and
@@ -702,13 +706,11 @@ class DataFrameCurator(SlotsCurator):
     ) -> None:
         super().__init__(dataset=dataset, schema=schema)
 
-        # Create atomic curator for features only
-        if len(self._schema.features.all()) > 0:
-            self._atomic_curator = ComponentCurator(
-                dataset=dataset,
-                schema=schema,
-                slot=slot,
-            )
+        self._atomic_curator = ComponentCurator(
+            dataset=dataset,
+            schema=schema,
+            slot=slot,
+        )
 
         # Handle (nested) attrs
         if slot is None and schema.slots:
@@ -724,11 +726,11 @@ class DataFrameCurator(SlotsCurator):
                             data = _resolve_schema_slot_path(
                                 attrs_dict, deeper_keys, slot_name, "attrs"
                             )
-                        df = pd.DataFrame([data])
+                        df = convert_dict_to_dataframe_for_validation(data, slot_schema)
                         self._slots[slot_name] = ComponentCurator(
                             df, slot_schema, slot=slot_name
                         )
-                else:
+                elif slot_name != "__external__":
                     raise ValueError(
                         f"Slot '{slot_name}' is not supported for DataFrameCurator. Must be 'attrs'."
                     )
@@ -783,6 +785,26 @@ class DataFrameCurator(SlotsCurator):
             )
 
 
+class ExperimentalDictCurator(DataFrameCurator):
+    """Curator for `dict` based on `DataFrameCurator`."""
+
+    def __init__(
+        self,
+        dataset: dict | Artifact,
+        schema: Schema,
+        slot: str | None = None,
+    ) -> None:
+        if not isinstance(dataset, dict) and not isinstance(dataset, Artifact):
+            raise InvalidArgument("The dataset must be a dict or dict-like artifact.")
+        if isinstance(dataset, Artifact):
+            assert dataset.otype == "dict", "Artifact must be of otype 'dict'."  # noqa: S101
+            d = dataset.load(is_run_input=False)
+        else:
+            d = dataset
+        df = convert_dict_to_dataframe_for_validation(d, schema)
+        super().__init__(df, schema, slot=slot)
+
+
 def _resolve_schema_slot_path(
     target_dict: dict[str, Any], slot_keys: Iterable[str], slot: str, base_path: str
 ) -> Any:
@@ -803,13 +825,18 @@ def _resolve_schema_slot_path(
         base_path += f"['{key}']"
         try:
             current = current[key]
-        except KeyError:
+        except (
+            KeyError,
+            TypeError,
+        ):  # if not a dict, raises TypeError; if a dict and key not found, raises KeyError
             available = (
-                list(current.keys()) if isinstance(current, dict) else "not a dict"
+                list(current.keys())
+                if isinstance(current, dict)
+                else "none (not a dict)"
             )
             raise InvalidArgument(
                 f"Schema slot '{slot}' requires keys {base_path} but key '{key}' "
-                f"not found. Available keys at this level: {available}"
+                f"not found. Available keys at this level: {available}."
             ) from None
 
     return current
@@ -1237,9 +1264,12 @@ class CatVector:
         source: SQLRecord | None = None,  # The ontology source to validate against.
         feature: Feature | None = None,
         cat_manager: DataFrameCatManager | None = None,
-        subtype_str: str = "",
+        filter_str: str = "",
+        subtypes_list: list[str] = None,
         maximal_set: bool = True,  # whether unvalidated categoricals cause validation failure.
     ) -> None:
+        if subtypes_list is None:
+            subtypes_list = []
         self._values_getter = values_getter
         self._values_setter = values_setter
         self._field = field
@@ -1249,22 +1279,38 @@ class CatVector:
         self._validated: None | list[str] = None
         self._non_validated: None | list[str] = None
         self._synonyms: None | dict[str, str] = None
-        self._subtype_str = subtype_str
+        self._filter_str = filter_str
+        self._subtypes_list = subtypes_list
         self._subtype_query_set = None
         self._cat_manager = cat_manager
         self.feature = feature
         self.records = None
         self._maximal_set = maximal_set
+        self._type_record = None
 
         self._all_filters = {"source": self._source, "organism": self._organism}
 
-        if self._subtype_str and "=" in self._subtype_str:
+        if self._filter_str:
             self._all_filters.update(
                 resolve_relation_filters(
-                    parse_filter_string(self._subtype_str), self._field.field.model
+                    parse_filter_string(self._filter_str), self._field.field.model
                 )  # type: ignore
             )
 
+        # get the dtype associated record based on the nested subtypes
+        if self._subtypes_list:
+            type_filters = {"name": self._subtypes_list[-1]}
+            if len(self._subtypes_list) > 1:
+                for i, nested_subtype in enumerate(reversed(self._subtypes_list[:-1])):
+                    filter_key = f"{'type__' * (i + 1)}name"
+                    type_filters[filter_key] = nested_subtype
+            try:
+                self._type_record = self._field.field.model.get(**type_filters)
+            except Exception as e:
+                raise InvalidArgument(
+                    f"Error retrieving type record with filters {type_filters} for field {self._field.field.name}."
+                ) from e
+
         if hasattr(field.field.model, "_name_field"):
             label_ref_is_name = field.field.name == field.field.model._name_field
         else:
@@ -1384,13 +1430,12 @@ class CatVector:
         str_values = _flatten_unique(values)
 
         # inspect the default instance and save validated records from public
-        if self._subtype_str != "" and "=" not in self._subtype_str:
+        if self._type_record is not None:
             related_name = registry._meta.get_field("type").remote_field.related_name
-            type_record = registry.get(name=self._subtype_str)
             if registry.__name__ == "Record":
-                self._subtype_query_set = type_record.query_records()
+                self._subtype_query_set = self._type_record.query_records()
             else:
-                self._subtype_query_set = getattr(type_record, related_name).all()
+                self._subtype_query_set = getattr(self._type_record, related_name).all()
             values_array = np.array(str_values)
             validated_mask = self._subtype_query_set.validate(  # type: ignore
                 values_array, field=self._field, **filter_kwargs, mute=True
@@ -1456,9 +1501,6 @@ class CatVector:
         registry = self._field.field.model
         field_name = self._field.field.name
         non_validated_records: SQLRecordList[Any] = []  # type: ignore
-        type_record = None
-        if self._subtype_str != "" and "=" not in self._subtype_str:
-            type_record = registry.get(name=self._subtype_str)
         if df is not None and registry == Feature:
             nonval_columns = Feature.inspect(df.columns, mute=True).non_validated
             non_validated_records = Feature.from_dataframe(df.loc[:, nonval_columns])
@@ -1475,10 +1517,13 @@ class CatVector:
                 init_kwargs = {field_name: value}
                 if registry == Feature:
                     init_kwargs["dtype"] = "cat" if dtype is None else dtype
-                if type_record is not None:
-                    # if subtype_str is set, we need to set the type for new records
-                    init_kwargs["type"] = type_record
-                non_validated_records.append(registry(**init_kwargs, **create_kwargs))
+                if self._type_record is not None:
+                    # if type_record is set, we need to set the type for new records
+                    init_kwargs["type"] = self._type_record
+                # here we create non-validated records skipping validation since we already ensured that they don't exist
+                non_validated_records.append(
+                    registry(**init_kwargs, **create_kwargs, _skip_validation=True)
+                )
         if len(non_validated_records) > 0:
             ln_save(non_validated_records)
             model_field = colors.italic(registry.__get_name_with_module__())
@@ -1574,7 +1619,7 @@ class CatVector:
                         check_organism = f"fix organism '{organism}', "
                 warning_message += f"    → {check_organism}fix typos, remove non-existent values, or save terms via: {colors.cyan(non_validated_hint_print)}"
                 if self._subtype_query_set is not None:
-                    warning_message += f"\n    → a valid label for subtype '{self._subtype_str}' has to be one of {self._subtype_query_set.to_list('name')}"
+                    warning_message += f"\n    → a valid label for subtype '{self._subtypes_list[-1]}' has to be one of {self._subtype_query_set.to_list('name')}"
             logger.info(f'mapping "{self._key}" on {colors.italic(model_field)}')
             logger.warning(warning_message)
             if self._cat_manager is not None:
@@ -1680,8 +1725,6 @@ class DataFrameCatManager:
                 0
             ]  # TODO: support composite dtypes for categoricals
             key = feature.name
-            field = result["field"]
-            subtype_str = result["subtype_str"]
             self._cat_vectors[key] = CatVector(
                 values_getter=lambda k=key: self._dataset[
                     k
@@ -1689,23 +1732,28 @@ class DataFrameCatManager:
                 values_setter=lambda new_values, k=key: self._dataset.__setitem__(
                     k, new_values
                 ),
-                field=field,
+                field=result["field"],
                 key=key,
                 source=self._sources.get(key),
                 feature=feature,
                 cat_manager=self,
-                subtype_str=subtype_str,
+                filter_str=result["filter_str"],
+                subtypes_list=result["subtypes_list"],
             )
         if index is not None and index.dtype.startswith("cat"):
             result = parse_dtype(index.dtype)[0]
-            field = result["field"]
             key = "index"
             self._cat_vectors[key] = CatVector(
                 values_getter=self._dataset.index,
-                field=field,
+                values_setter=lambda new_values: setattr(
+                    self._dataset, "index", new_values
+                ),
+                field=result["field"],
                 key=key,
                 feature=index,
                 cat_manager=self,
+                filter_str=result["filter_str"],
+                subtypes_list=result["subtypes_list"],
             )
 
     @property
@@ -1975,7 +2023,8 @@ def annotate_artifact(
 
     slug = ln_setup.settings.instance.slug
     if ln_setup.settings.instance.is_remote:  # pdagma: no cover
-        logger.important(f"go to https://lamin.ai/{slug}/artifact/{artifact.uid}")
+        ui_url = ln_setup.settings.instance.ui_url
+        logger.important(f"go to {ui_url}/{slug}/artifact/{artifact.uid}")
     return artifact
 
 
