@@ -6,7 +6,7 @@ from typing import TYPE_CHECKING, Any, Literal, overload
 import anndata as ad
 import pandas as pd
 from django.db import models
-from django.db.models import CASCADE, PROTECT
+from django.db.models import CASCADE, PROTECT, Q
 from lamin_utils import logger
 from lamindb_setup.core.hashing import HASH_LENGTH, hash_set
 
@@ -25,11 +25,11 @@ from ..models._is_versioned import process_revises
 from ._is_versioned import IsVersioned
 from .artifact import (
     Artifact,
-    _populate_subsequent_runs_,
-    _track_run_input,
     describe_artifact_collection,
     get_run,
+    populate_subsequent_run,
     save_schema_links,
+    track_run_input,
 )
 from .has_parents import view_lineage
 from .run import Run, TracksRun, TracksUpdates
@@ -155,6 +155,12 @@ class Collection(SQLRecord, IsVersioned, TracksRun, TracksUpdates):
     class Meta(SQLRecord.Meta, IsVersioned.Meta, TracksRun.Meta, TracksUpdates.Meta):
         abstract = False
         app_label = "lamindb"
+        constraints = [
+            models.UniqueConstraint(
+                fields=["key", "hash"],
+                name="unique_collection_key_hash_not_null",
+            )
+        ]
 
     _len_full_uid: int = 20
     _len_stem_uid: int = 16
@@ -178,7 +184,9 @@ class Collection(SQLRecord, IsVersioned, TracksRun, TracksUpdates):
     description: str | None = TextField(null=True)
     """A description or title."""
     hash: str | None = CharField(
-        max_length=HASH_LENGTH, db_index=True, null=True, unique=True
+        max_length=HASH_LENGTH,
+        db_index=True,
+        null=True,
     )
     """Hash of collection content."""
     reference: str | None = CharField(max_length=255, db_index=True, null=True)
@@ -189,7 +197,7 @@ class Collection(SQLRecord, IsVersioned, TracksRun, TracksUpdates):
     ulabels: ULabel = models.ManyToManyField(
         "ULabel", through="CollectionULabel", related_name="collections"
     )
-    """ULabels sampled in the collection (see :class:`~lamindb.Feature`)."""
+    """ULabels annotating the collection (see :class:`~lamindb.Feature`)."""
     run: Run | None = ForeignKey(
         Run, PROTECT, related_name="output_collections", null=True, default=None
     )
@@ -239,6 +247,7 @@ class Collection(SQLRecord, IsVersioned, TracksRun, TracksUpdates):
         reference_type: str | None = None,
         run: Run | None = None,
         revises: Collection | None = None,
+        skip_hash_lookup: bool = False,
     ): ...
 
     @overload
@@ -269,6 +278,7 @@ class Collection(SQLRecord, IsVersioned, TracksRun, TracksUpdates):
         run: Run | None = kwargs.pop("run", None)
         revises: Collection | None = kwargs.pop("revises", None)
         version: str | None = kwargs.pop("version", None)
+        skip_hash_lookup: bool = kwargs.pop("skip_hash_lookup", False)
         branch = kwargs.pop("branch", None)
         branch_id = kwargs.pop("branch_id", 1)
         space = kwargs.pop("space", None)
@@ -316,18 +326,20 @@ class Collection(SQLRecord, IsVersioned, TracksRun, TracksUpdates):
                         "Save meta_artifact artifact before creating collection!"
                     )
         # we ignore collections in trash containing the same hash
-        if hash is not None:
-            existing_collection = Collection.filter(hash=hash).one_or_none()
+        if hash is not None and not skip_hash_lookup:
+            existing_collection = Collection.objects.filter(
+                ~Q(branch_id=-1),
+                hash=hash,
+            ).one_or_none()
         else:
             existing_collection = None
         if existing_collection is not None:
             logger.warning(
-                f"returning existing collection with same hash: {existing_collection}; if you intended to query to track this collection as an input, use: ln.Collection.get()"
+                f"returning collection with same hash: {existing_collection}; if you intended to query to track this collection as an input, use: ln.Collection.get()"
             )
-            if run is not None:
-                existing_collection._populate_subsequent_runs(run)
             init_self_from_db(self, existing_collection)
             update_attributes(self, {"description": description, "key": key})
+            populate_subsequent_run(self, run)
         else:
             _skip_validation = revises is not None and key == revises.key
             super().__init__(  # type: ignore
@@ -348,9 +360,9 @@ class Collection(SQLRecord, IsVersioned, TracksRun, TracksUpdates):
                 _skip_validation=_skip_validation,
             )
         self._artifacts = artifacts
-        if revises is not None:
-            _track_run_input(revises, run=run)
-        _track_run_input(artifacts, run=run)
+        if revises is not None and revises.uid != self.uid:
+            track_run_input(revises, run=run)
+        track_run_input(artifacts, run=run)
 
     @classmethod
     def get(
@@ -440,7 +452,7 @@ class Collection(SQLRecord, IsVersioned, TracksRun, TracksUpdates):
 
         dataframe = _open_dataframe(paths, engine=engine, **kwargs)
         # track only if successful
-        _track_run_input(self, is_run_input)
+        track_run_input(self, is_run_input)
         return dataframe
 
     def mapped(
@@ -540,7 +552,7 @@ class Collection(SQLRecord, IsVersioned, TracksRun, TracksUpdates):
             dtype,
         )
         # track only if successful
-        _track_run_input(self, is_run_input)
+        track_run_input(self, is_run_input)
         return ds
 
     def cache(self, is_run_input: bool | None = None) -> list[UPath]:
@@ -557,7 +569,7 @@ class Collection(SQLRecord, IsVersioned, TracksRun, TracksUpdates):
         for artifact in self.ordered_artifacts.all():
             # do not want to track data lineage on the artifact level
             path_list.append(artifact.cache(is_run_input=False))
-        _track_run_input(self, is_run_input)
+        track_run_input(self, is_run_input)
         return path_list
 
     def load(
@@ -570,11 +582,11 @@ class Collection(SQLRecord, IsVersioned, TracksRun, TracksUpdates):
 
         Returns an in-memory concatenated `DataFrame` or `AnnData` object.
         """
-        # cannot call _track_run_input here, see comment further down
+        # cannot call track_run_input here, see comment further down
         artifacts = self.ordered_artifacts.all()
         concat_object = _load_concat_artifacts(artifacts, join, **kwargs)
         # only call it here because there might be errors during load or concat
-        _track_run_input(self, is_run_input)
+        track_run_input(self, is_run_input)
         return concat_object
 
     def save(self, using: str | None = None) -> Collection:
@@ -664,9 +676,6 @@ class Collection(SQLRecord, IsVersioned, TracksRun, TracksUpdates):
         """
         return describe_artifact_collection(self)
 
-    def _populate_subsequent_runs(self, run: Run) -> None:
-        _populate_subsequent_runs_(self, run)
-
 
 # internal function, not exposed to user
 def from_artifacts(artifacts: Iterable[Artifact]) -> tuple[str, dict[str, str]]:
