@@ -7,6 +7,7 @@ from typing import TYPE_CHECKING, Literal
 import lamindb_setup as ln_setup
 from lamin_utils import logger
 
+from .query_set import QuerySet, SQLRecordList
 from .run import Run
 from .sqlrecord import format_field_value, get_name_field
 
@@ -17,7 +18,6 @@ if TYPE_CHECKING:
 
     from .artifact import Artifact
     from .collection import Collection
-    from .query_set import QuerySet
     from .sqlrecord import SQLRecord
     from .transform import Transform
 
@@ -60,6 +60,20 @@ def _query_relatives(
     return query_relatives_on_branches(records, attr, cls)
 
 
+def _query_ancestors_of_fk(record: SQLRecord, attr: str) -> SQLRecordList:
+    from .query_set import get_default_branch_ids
+
+    branch_ids = get_default_branch_ids()
+    ancestors = []
+
+    current = getattr(record, attr)
+    while current is not None and current.branch_id in branch_ids:
+        ancestors.append(current)
+        current = getattr(current, attr)
+
+    return SQLRecordList(ancestors)
+
+
 class HasParents:
     """Base class for hierarchical registries (ontologies)."""
 
@@ -175,20 +189,13 @@ def view_digraph(u: Digraph):
 def view_lineage(
     data: Artifact | Collection, with_children: bool = True, return_graph: bool = False
 ) -> Digraph | None:
-    """Graph of data flow.
-
-    Notes:
-        For more info, see use cases: :doc:`docs:data-flow`.
-
-    Examples:
-        >>> collection.view_lineage()
-        >>> artifact.view_lineage()
-    """
+    """View data lineage graph."""
     if ln_setup.settings.instance.is_on_hub:
         instance_slug = ln_setup.settings.instance.slug
+        ui_url = ln_setup.settings.instance.ui_url
         entity_slug = data.__class__.__name__.lower()
         logger.important(
-            f"explore at: https://lamin.ai/{instance_slug}/{entity_slug}/{data.uid}"
+            f"explore at: {ui_url}/{instance_slug}/{entity_slug}/{data.uid}"
         )
 
     import graphviz
@@ -550,9 +557,7 @@ def _get_all_child_runs(data: Artifact | Collection) -> list:
         runs.update(
             {
                 f.run
-                for f in data.run.output_collections.all()
-                .filter(branch_id__in=[0, 1])
-                .all()
+                for f in data.run.output_collections.all().filter(branch_id__in=[0, 1])
             }
         )
     while runs.difference(all_runs):
