@@ -23,6 +23,7 @@ import lamindb_setup as ln_setup
 from django.core.exceptions import ValidationError as DjangoValidationError
 from django.db import IntegrityError, ProgrammingError, connections, models, transaction
 from django.db.models import CASCADE, PROTECT, Field, Manager, QuerySet
+from django.db.models import ForeignKey as django_ForeignKey
 from django.db.models.base import ModelBase
 from django.db.models.fields.related import (
     ManyToManyField,
@@ -186,7 +187,7 @@ def update_attributes(record: SQLRecord, attributes: dict[str, str]):
                     if key == "hash"
                     else f"keeping {getattr(record, key)}"
                 )
-                logger.warning(
+                logger.debug(
                     f"ignoring tentative value {value} for {key}, {hash_message}"
                 )
 
@@ -305,11 +306,18 @@ def suggest_records_with_similar_names(
     # but this isn't reliable: https://laminlabs.slack.com/archives/C04FPE8V01W/p1737812808563409
     # the below needs to be .first() because there might be multiple records with the same
     # name field in case the record is versioned (e.g. for Transform key)
-    exact_match = record.__class__.filter(**{name_field: kwargs[name_field]}).first()
+    if hasattr(record.__class__, "type"):
+        if kwargs.get("type", None) is None:
+            subset = record.__class__.filter(type__isnull=True)
+        else:
+            subset = record.__class__.filter(type=kwargs["type"])
+    else:
+        subset = record.__class__
+    exact_match = subset.filter(**{name_field: kwargs[name_field]}).first()
     if exact_match is not None:
         return exact_match
     queryset = _search(
-        record.__class__,
+        subset,
         kwargs[name_field],
         field=name_field,
         truncate_string=True,
@@ -687,17 +695,17 @@ class Registry(ModelBase):
 
     def __get_available_fields__(cls) -> set[str]:
         if cls._available_fields is None:
-            cls._available_fields = {
-                f.name
-                for f in cls._meta.get_fields()
-                if not f.name.startswith("_")
-                and not f.name.startswith("links_")
-                and not f.name.endswith("_id")
-            }
+            available_fields = set()
+            for field in cls._meta.get_fields():
+                if not (field_name := field.name).startswith(("_", "links_")):
+                    available_fields.add(field_name)
+                    if isinstance(field, django_ForeignKey):
+                        available_fields.add(field_name + "_id")
             if cls.__name__ == "Artifact":
-                cls._available_fields.add("visibility")  # backward compat
-                cls._available_fields.add("_branch_code")  # backward compat
-                cls._available_fields.add("transform")
+                available_fields.add("visibility")  # backward compat
+                available_fields.add("_branch_code")  # backward compat
+                available_fields.add("transform")
+            cls._available_fields = available_fields
         return cls._available_fields
 
 
@@ -812,7 +820,7 @@ class BaseSQLRecord(models.Model, metaclass=Registry):
                             existing_record = exact_match
                         if existing_record is not None:
                             logger.important(
-                                f"returning existing {self.__class__.__name__} record with same"
+                                f"returning {self.__class__.__name__.lower()} with same"
                                 f" {name_field}{version_comment}: '{kwargs[name_field]}'"
                             )
                             init_self_from_db(self, existing_record)
@@ -894,7 +902,7 @@ class BaseSQLRecord(models.Model, metaclass=Registry):
                 # "duplicate key value violates unique constraint"
                 # "UNIQUE constraint failed"
                 if (
-                    self.__class__.__name__ in {"Transform", "Artifact"}
+                    self.__class__.__name__ in {"Transform", "Artifact", "Collection"}
                     and isinstance(e, IntegrityError)
                     and "hash" in error_msg
                     and unique_constraint_error_in_error_message(error_msg)
@@ -942,7 +950,7 @@ class BaseSQLRecord(models.Model, metaclass=Registry):
                     )
                     if pre_existing_record.branch_id == 1:
                         logger.warning(
-                            f"returning existing {self.__class__.__name__} record with same {', '.join(constraint_fields)}: '{', '.join([str(getattr(self, f)) for f in constraint_fields])}'"
+                            f"returning {self.__class__.__name__} record with same {', '.join(constraint_fields)}: '{', '.join([str(getattr(self, f)) for f in constraint_fields])}'"
                         )
                     else:
                         # modifies the fields of the existing record with new values of self
@@ -957,12 +965,32 @@ class BaseSQLRecord(models.Model, metaclass=Registry):
                     init_self_from_db(self, pre_existing_record)
                 elif (
                     isinstance(e, ProgrammingError)
-                    and hasattr(self, "space")
                     and "new row violates row-level security policy" in error_msg
+                    and (
+                        (is_locked := getattr(self, "is_locked", False))
+                        or hasattr(self, "space")
+                    )
                 ):
+                    if is_locked:
+                        no_write_msg = "It is not allowed to modify or create locked ('is_locked=True') records."
+                    else:
+                        no_write_msg = (
+                            f"You're not allowed to write to the space '{self.space.name}'.\n"
+                            "Please contact administrators of the space if you need write access."
+                        )
+                    raise NoWriteAccess(no_write_msg) from None
+                elif (
+                    isinstance(e, ProgrammingError)
+                    and "permission denied for table" in error_msg
+                    and (isettings := setup_settings.instance)._db_permissions
+                    == "public"
+                ):
+                    slug = isettings.slug
                     raise NoWriteAccess(
-                        f"You're not allowed to write to the space '{self.space.name}'.\n"
-                        "Please contact administrators of the space if you need write access."
+                        f"You are trying to write to '{slug}' with public (read-only) permissions.\n"
+                        "Please contact administrators to make you a collaborator if you need write access.\n"
+                        f"If you are already a collaborator, please do 'lamin connect {slug}' in console, "
+                        "restart the python session and try again."
                     ) from None
                 else:
                     raise
@@ -1224,6 +1252,11 @@ class SQLRecord(BaseSQLRecord, metaclass=Registry):
     def _branch_code(self, value: int):
         self.branch_id = value
 
+    def restore(self) -> None:
+        """Restore from trash onto the main branch."""
+        self.branch_id = 1
+        self.save()
+
     def delete(self, permanent: bool | None = None, **kwargs) -> None:
         """Delete record.
 
@@ -1433,7 +1466,7 @@ def add_db_connection(db: str, using: str):
     connections.settings[using] = db_config
 
 
-REGISTRY_UNIQUE_FIELD = {"storage": "root", "feature": "name", "ulabel": "name"}
+REGISTRY_UNIQUE_FIELD = {"storage": "root", "ulabel": "name"}
 
 
 def update_fk_to_default_db(
