@@ -73,6 +73,7 @@ def install(session):
         "faq",
         "storage",
         "curator",
+        "integrations",
         "docs",
         "cli",
         "permissions",
@@ -131,6 +132,8 @@ def install_ci(session, group):
             "uv pip install --system spatialdata",
         )
         run(session, "uv pip install --system tiledbsoma")
+    elif group == "integrations":
+        run(session, "uv pip install --system lightning")
     elif group == "docs":
         extras += "zarr"
         # spatialdata dependency, specifying it here explicitly
@@ -169,6 +172,8 @@ def install_ci(session, group):
             session,
             "uv pip install --system -e ./laminhub/rest-hub --no-build-isolation",
         )
+    # needed here till the next release of lamindb-setup
+    run(session, "uv pip install --system httpx_retries")
     # on the release branch, do not use submodules but run with pypi install
     # only exception is the docs group which should always use the submodule
     # to push docs fixes fast
@@ -221,6 +226,7 @@ def configure_coverage(session) -> None:
         "unit-core",
         "unit-storage",
         "curator",
+        "integrations",
         "tutorial",
         "guide",
         "biology",
@@ -268,6 +274,8 @@ def test(session, group):
             session,
             f"pytest {coverage_args} tests/curators {duration_args}",
         )
+    elif group == "integrations":
+        run(session, f"pytest -s {coverage_args} tests/integrations")
     elif group == "cli":
         run(
             session,
